#include "drawarea.h"
#include <gdk/gdkkeysyms.h>

DrawArea::DrawArea(){
  state=0;
  imagePX=NULL;
  imagePY=NULL;
  imageRotate=NULL;
  imageSize=NULL;
  tPX=0;
  tPY=0;
  dPX=0;
  dPY=0;
  fullFlag=0;
  
  //set_events(Gdk::ALL_EVENTS_MASK);
  
  add_events(Gdk::FOCUS_CHANGE_MASK);
  add_events(Gdk::BUTTON_PRESS_MASK);
  add_events(Gdk::BUTTON_RELEASE_MASK);
  add_events(Gdk::BUTTON_MOTION_MASK);
  add_events(Gdk::KEY_PRESS_MASK);
  add_events(Gdk::KEY_RELEASE_MASK); 
  add_events(Gdk::SCROLL_MASK);
  
  property_can_focus()=true;
  //  can_focus();
}

DrawArea::~DrawArea(){
  
}

void DrawArea::Create(Gtk::Window *window0,std::string filename0,
		      double *imagePX0,double *imagePY0,
		      double *imageRotate0,double *imageSize0){
  int value;

  imagePX=imagePX0;
  imagePY=imagePY0;
  imageRotate=imageRotate0;
  imageSize=imageSize0;

  parWin=window0;

  /*
  signal_key_press_event().connect
    (sigc::mem_fun(*this,&DrawArea::KeyPress),false);
  */

  grab_focus();
  has_focus();
  can_focus();
  
  fileList.ReadFolder(filename0);

  SetImageVariable(filename0);

  std::cout<<"DRAWAREA CREATE"<<std::endl;
}

void DrawArea::SetImageVariable(std::string filename0){
  double powW,powH;
  int maxWidth,maxHeight;
  int flag0;

  flag0=drawImage.SetImage(fileList.GetTargetPath(),
			   fileList.GetTargetType());

  if(flag0){
    *imageSize=1;
    
    maxWidth=parWin->get_screen()->get_width();
    maxHeight=parWin->get_screen()->get_height();
    
    std::cout<<"MAX WIDTH :"<<maxWidth<<"\t";
    std::cout<<"MAX HEIGHT:"<<maxHeight<<std::endl;
    
    powW=(double)maxWidth/drawImage.GetWidth();
    powH=(double)maxHeight/drawImage.GetHeight();
    
    if(maxWidth<drawImage.GetWidth()){
      if(maxHeight<drawImage.GetHeight()){
	if(powW<powH){
	  *imageSize=powW;
	}
	else{
	  *imageSize=powH;
	}
	parWin->resize(maxWidth,maxHeight);
      }
      else{
	*imageSize=powW;
	parWin->resize(maxHeight,drawImage.GetHeight());
      }
    }
    else{
      if(maxHeight<drawImage.GetHeight()){
	*imageSize=powH;
	parWin->resize(drawImage.GetWidth(),maxHeight);
      }
      else{
	parWin->resize(drawImage.GetWidth(),drawImage.GetHeight());
      }
    }
  }
  else{
    std::cout<<"NO IMAGE"<<std::endl;
    parWin->resize(400,400);
  }

  imageChange.emit
    (fileList.GetTarget(),drawImage.GetWidth(),drawImage.GetHeight());
}

void DrawArea::SetImageFixed(std::string filename0){
  Gdk::Rectangle rect(0,0,get_width(),get_height());
  int flag0=0;
  double powW,powH;

  flag0=drawImage.SetImage(fileList.GetTargetPath(),
			   fileList.GetTargetType());

  if(flag0){
    
    powW=(double)get_width()/(double)drawImage.GetWidth();
    powH=(double)get_height()/(double)drawImage.GetHeight();
    
    if(powW<powH){
      *imageSize=powW;
    }
    else{
      *imageSize=powH;
    }
    *imagePX=(double)get_width()/2;
    *imagePY=(double)get_height()/2;
    *imageRotate=0;
  }
  else{
    std::cout<<"NO IMAGE"<<std::endl;
  }

  imageChange.emit
    (fileList.GetTarget(),drawImage.GetWidth(),drawImage.GetHeight());
  sizeChange.emit(*imageSize);
  rotateChange.emit(*imageRotate);

  get_window()->invalidate_rect(rect,false);
}

void DrawArea::OpenImage(std::string filename0){
  fileList.ReadFolder(filename0);
  fileList.View();
  SetImageFixed(filename0);
}

void DrawArea::NextImage(){
  fileList.NextTarget();
  SetImageFixed(fileList.GetTargetPath());
}

void DrawArea::PrevImage(){
  fileList.PrevTarget();
  SetImageFixed(fileList.GetTargetPath());
}

void DrawArea::FitImage(){
  double powW,powH;
  Gdk::Rectangle rect(0,0,get_width(),get_height());

  if(drawImage.GetWidth()!=0&&drawImage.GetHeight()!=0){
    powW=(double)rect.get_width()/(double)drawImage.GetWidth();
    powH=(double)rect.get_height()/(double)drawImage.GetHeight();
    if(powW<powH){
      *imageSize=powW;
    }
    else{
      *imageSize=powH;
    } 
  }
  else{
    *imageSize=-1;
  }

  *imageRotate=0;
  sizeChange.emit(*imageSize);
  rotateChange.emit(*imageRotate);
  
  *imagePX=(double)rect.get_width()/2;
  *imagePY=(double)rect.get_height()/2;

  get_window()->invalidate_rect(rect,false);
}

std::string DrawArea::GetCurrentFolder(){
  return fileList.GetCurrentFolder();
}

void DrawArea::on_realize(){
  std::cout<<"RERALIZE"<<std::endl;
  Gtk::DrawingArea::on_realize();
  *imagePX=(double)get_width()/2;
  *imagePY=(double)get_height()/2;
}

void DrawArea::on_size_allocate(Gdk::Rectangle &rect){
  double powW,powH;
  Gtk::Widget::on_size_allocate(rect);

  if(drawImage.GetWidth()!=0&&drawImage.GetHeight()!=0){
    powW=(double)rect.get_width()/(double)drawImage.GetWidth();
    powH=(double)rect.get_height()/(double)drawImage.GetHeight();
    if(powW<powH){
      *imageSize=powW;
    }
    else{
      *imageSize=powH;
    } 
  }
  else{
    *imageSize=-1;
  }

  sizeChange.emit(*imageSize);
  
  *imagePX=(double)rect.get_width()/2;
  *imagePY=(double)rect.get_height()/2;
}

bool DrawArea::on_expose_event(GdkEventExpose *event){
  Cairo::Matrix matrix0;
  
  drawContext=get_window()->create_cairo_context();
  
  drawContext->set_source_rgb(0,0,0);
  drawContext->paint();
  
  drawContext->get_matrix(matrix0);
  
  cairo_matrix_translate(&matrix0,*imagePX,*imagePY);
  cairo_matrix_scale(&matrix0,*imageSize,*imageSize);
  cairo_matrix_rotate(&matrix0,*imageRotate);
  drawContext->set_matrix(matrix0);

  drawImage.DrawContext(drawContext);  
  return true;
}

bool DrawArea::on_button_press_event(GdkEventButton *event){
  property_has_focus()=true;

  if(event->button==3){
    if(fullFlag){
      fullChange.emit(false);
      fullFlag=0;
    }
  }
  
  tButton=event->button;
  tRotate=*imageRotate;
  tPX=*imagePX;
  tPY=*imagePY;
  dPX=event->x;
  dPY=event->y;
  
  return true;
}

bool DrawArea::on_button_release_event(GdkEventButton *event){
  
  tButton=0;
  tRotate=0;
  tPX=0;
  tPY=0;
  dPX=0;
  dPY=0;
  
  return true;
}

bool DrawArea::on_motion_notify_event(GdkEventMotion *event){
  Gdk::Rectangle rect(0,0,get_width(),get_height());
  
  if(tButton==1){
    *imagePX=tPX+(event->x-dPX);
    *imagePY=tPY+(event->y-dPY);
  }
  else if(tButton==2){
    *imageRotate=tRotate+(event->x-dPX)*0.01;
    if(*imageRotate>M_PI*2){
      *imageRotate-=M_PI*2;
    }
    else if(*imageRotate<=0){
      *imageRotate+=M_PI*2;
    }
    rotateChange.emit(*imageRotate);
  }
  get_window()->invalidate_rect(rect,false);
  return true;
}

bool DrawArea::on_scroll_event(GdkEventScroll *event){
  Gdk::Rectangle rect(0,0,get_width(),get_height());
  
  if(event->direction==GDK_SCROLL_UP){
    *imageSize+=0.1;
  }
  else if(event->direction==GDK_SCROLL_DOWN){
    *imageSize-=0.1;
  }

  sizeChange.emit(*imageSize);

  get_window()->invalidate_rect(rect,false);
  return true;
}

bool DrawArea::on_key_press_event(GdkEventKey *event){
  Gdk::Rectangle rect(0,0,get_width(),get_height());
  std::cout<<"KEY PRESS:"<<" "
	   <<"TYPE="<<event->type<<" "
	   <<"STATE="<<event->state<<" "
	   <<"VALUE="<<std::hex<<event->keyval<<" "
	   <<"STRING="<<event->string<<" "
	   <<"KEYCODE="<<event->hardware_keycode<<" "
	   <<std::endl;

  if(event->state==GDK_SHIFT_MASK){
    std::cout<<"SHIFT MASK"<<std::endl;
  }
  else if(event->state==GDK_LOCK_MASK){
    std::cout<<"LOCK MASK"<<std::endl;
  }
  else if(event->state==GDK_CONTROL_MASK){
    std::cout<<"CONTROL MASK"<<std::endl;
    if(event->keyval==GDK_Left){
      *imageRotate-=0.1;
      if(*imageRotate>M_PI*2){
	*imageRotate-=M_PI*2;
      }
      else if(*imageRotate<=0){
	*imageRotate+=M_PI*2;
      }
      rotateChange.emit(*imageRotate);
    }
    else if(event->keyval==GDK_Right){
      *imageRotate+=0.1;
      if(*imageRotate>M_PI*2){
	*imageRotate-=M_PI*2;
      }
      else if(*imageRotate<=0){
	*imageRotate+=M_PI*2;
      }
      rotateChange.emit(*imageRotate);
    }
    else if(event->keyval==GDK_Up){
      *imageSize+=0.1;
      sizeChange.emit(*imageSize);
    }
    else if(event->keyval==GDK_Down){
      *imageSize-=0.1;
      sizeChange.emit(*imageSize);
    }

    get_window()->invalidate_rect(rect,false);
  }
  else if(event->state==GDK_MOD1_MASK){
    std::cout<<"ALT MASK"<<std::endl;    
    
    if(event->keyval==GDK_Left){
      *imagePX-=10;
    }
    if(event->keyval==GDK_Right){
      *imagePX+=10;
    }
    if(event->keyval==GDK_Up){
      *imagePY-=10;
    }
    if(event->keyval==GDK_Down){
      *imagePY+=10;
    }

    get_window()->invalidate_rect(rect,false);
  }
  else{
    if(event->keyval==GDK_Left){
      PrevImage();
    }
    else if(event->keyval==GDK_Right){
      NextImage();
    }
    else if(event->keyval==GDK_F1){
      std::cout<<"FULL"<<std::endl;
      if(fullFlag){
	fullChange.emit(false);
	fullFlag=0;
      }
      else{
	fullChange.emit(true);
	fullFlag=1;
      }
      /*
	parWin->fullscreen();
	get_window()->fullscreen();
	add_modal_grab();
      */
    }
    else if(event->keyval==GDK_space){
      FitImage();
    }
  }
}

bool DrawArea::on_key_release_event(GdkEventKey *event){
  /*
  std::cout<<"KEY RELEASE:"<<" "
	   <<"TYPE="<<event->type<<" "
	   <<"STATE="<<event->state<<" "
	   <<"VALUE="<<std::hex<<event->keyval<<" "
	   <<"STRING="<<event->string<<" "
	   <<"KEYCODE="<<event->hardware_keycode<<" "
	   <<std::endl;
  */
}
/*
void DrawArea::on_grab_focus(){
  std::cout<<"GRAB FOCUS"<<std::endl;
    property_can_focus()=true;
    property_has_focus()=true;
   add_modal_grab();
}

bool DrawArea::on_focus_in_event(GdkEventFocus *event){
  std::cout<<"GRAB FOCUS IN"<<std::endl;
}

bool DrawArea::on_focus_out_event(GdkEventFocus *event){
  std::cout<<"GRAB FOCUS OUT"<<std::endl;
}
*/
DrawArea::type_rotate DrawArea::SignalRotateChange(double value){
  return rotateChange;
}

DrawArea::type_size DrawArea::SignalSizeChange(double value){
  
  return sizeChange;
}

DrawArea::type_fullscreen DrawArea::SignalFullChange(bool value){
  return fullChange;
}

DrawArea::type_image DrawArea::SignalImageChange
(FileItem *target,double width,double height){
  return imageChange;
}
